@@ -12,19 +12,22 @@ from hyperliquid.utils.constants import MAINNET_API_URL
 from hyperliquid.utils.signing import (
     ZERO_ADDRESS,
     CancelRequest,
+    CancelByCloidRequest,
     OrderRequest,
     OrderSpec,
     OrderType,
     float_to_usd_int,
     get_timestamp_ms,
     order_grouping_to_number,
+    order_request_to_order_spec,
     order_spec_preprocessing,
     order_spec_to_order_wire,
     sign_l1_action,
     sign_usd_transfer_action,
     sign_agent,
+    str_to_bytes16,
 )
-from hyperliquid.utils.types import Any, List, Literal, Meta, Optional, Tuple
+from hyperliquid.utils.types import Any, List, Literal, Meta, Optional, Tuple, Cloid
 
 
 class Exchange(API):
@@ -56,42 +59,53 @@ class Exchange(API):
         return self.post("/exchange", payload)
 
     def order(
-        self, coin: str, is_buy: bool, sz: float, limit_px: float, order_type: OrderType, reduce_only: bool = False
+        self,
+        coin: str,
+        is_buy: bool,
+        sz: float,
+        limit_px: float,
+        order_type: OrderType,
+        reduce_only: bool = False,
+        cloid: Optional[Cloid] = None,
     ) -> Any:
-        return self.bulk_orders(
-            [
-                {
-                    "coin": coin,
-                    "is_buy": is_buy,
-                    "sz": sz,
-                    "limit_px": limit_px,
-                    "order_type": order_type,
-                    "reduce_only": reduce_only,
-                }
-            ]
-        )
+        order = {
+            "coin": coin,
+            "is_buy": is_buy,
+            "sz": sz,
+            "limit_px": limit_px,
+            "order_type": order_type,
+            "reduce_only": reduce_only,
+        }
+        if cloid:
+            order["cloid"] = cloid
+        return self.bulk_orders([order])
 
     def bulk_orders(self, order_requests: List[OrderRequest]) -> Any:
         order_specs: List[OrderSpec] = [
-            {
-                "order": {
-                    "asset": self.coin_to_asset[order["coin"]],
-                    "isBuy": order["is_buy"],
-                    "reduceOnly": order["reduce_only"],
-                    "limitPx": order["limit_px"],
-                    "sz": order["sz"],
-                },
-                "orderType": order["order_type"],
-            }
-            for order in order_requests
+            order_request_to_order_spec(order, self.coin_to_asset[order["coin"]]) for order in order_requests
         ]
 
         timestamp = get_timestamp_ms()
         grouping: Literal["na"] = "na"
 
+        has_cloid = False
+        for order_spec in order_specs:
+            if "cloid" in order_spec["order"] and order_spec["order"]["cloid"]:
+                has_cloid = True
+
+        if has_cloid:
+            for order_spec in order_specs:
+                if "cloid" not in order_spec["order"] or not order_spec["order"]["cloid"]:
+                    raise ValueError("all orders must have cloids if at least one has a cloid")
+
+        if has_cloid:
+            signature_types = ["(uint32,bool,uint64,uint64,bool,uint8,uint64,bytes16)[]", "uint8"]
+        else:
+            signature_types = ["(uint32,bool,uint64,uint64,bool,uint8,uint64)[]", "uint8"]
+
         signature = sign_l1_action(
             self.wallet,
-            ["(uint32,bool,uint64,uint64,bool,uint8,uint64)[]", "uint8"],
+            signature_types,
             [[order_spec_preprocessing(order_spec) for order_spec in order_specs], order_grouping_to_number(grouping)],
             ZERO_ADDRESS if self.vault_address is None else self.vault_address,
             timestamp,
@@ -111,6 +125,9 @@ class Exchange(API):
     def cancel(self, coin: str, oid: int) -> Any:
         return self.bulk_cancel([{"coin": coin, "oid": oid}])
 
+    def cancel_by_cloid(self, coin: str, cloid: Cloid) -> Any:
+        return self.bulk_cancel_by_cloid([{"coin": coin, "cloid": cloid}])
+
     def bulk_cancel(self, cancel_requests: List[CancelRequest]) -> Any:
         timestamp = get_timestamp_ms()
         signature = sign_l1_action(
@@ -136,6 +153,36 @@ class Exchange(API):
             timestamp,
         )
 
+    def bulk_cancel_by_cloid(self, cancel_requests: List[CancelByCloidRequest]) -> Any:
+        timestamp = get_timestamp_ms()
+        signature = sign_l1_action(
+            self.wallet,
+            ["(uint32,bytes16)[]"],
+            [
+                [
+                    (self.coin_to_asset[cancel["coin"]], str_to_bytes16(cancel["cloid"].to_raw()))
+                    for cancel in cancel_requests
+                ]
+            ],
+            ZERO_ADDRESS if self.vault_address is None else self.vault_address,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            {
+                "type": "cancelByCloid",
+                "cancels": [
+                    {
+                        "asset": self.coin_to_asset[cancel["coin"]],
+                        "cloid": cancel["cloid"],
+                    }
+                    for cancel in cancel_requests
+                ],
+            },
+            signature,
+            timestamp,
+        )
+
     def update_leverage(self, leverage: int, coin: str, is_cross: bool = True) -> Any:
         timestamp = get_timestamp_ms()
         asset = self.coin_to_asset[coin]
