@@ -1,11 +1,11 @@
 import sys
 
 if sys.version_info >= (3, 8):
-    from typing import TypedDict, Literal
+    from typing import Literal, TypedDict
 else:
     from typing_extensions import TypedDict, Literal
 
-from typing import List, Union, Dict, Tuple, Any, Optional, cast, Callable, NamedTuple
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple, Union, cast
 
 Any = Any
 Option = Optional
@@ -29,6 +29,7 @@ AllMidsMsg = TypedDict("AllMidsMsg", {"channel": Literal["allMids"], "data": All
 L2Level = TypedDict("L2Level", {"px": str, "sz": str, "n": int})
 L2BookData = TypedDict("L2BookData", {"coin": str, "levels": Tuple[List[L2Level]], "time": int})
 L2BookMsg = TypedDict("L2BookMsg", {"channel": Literal["l2Book"], "data": L2BookData})
+PongMsg = TypedDict("PongMsg", {"channel": Literal["pong"]})
 Trade = TypedDict("Trade", {"coin": str, "side": Side, "px": str, "sz": int, "hash": str, "time": int})
 TradesMsg = TypedDict("TradesMsg", {"channel": Literal["trades"], "data": List[Trade]})
 Fill = TypedDict(
@@ -50,4 +51,25 @@ Fill = TypedDict(
 # TODO: handle other types of user events
 UserEventsData = TypedDict("UserEventsData", {"fills": List[Fill]}, total=False)
 UserEventsMsg = TypedDict("UserEventsMsg", {"channel": Literal["user"], "data": UserEventsData})
-WsMsg = Union[AllMidsMsg, L2BookMsg, TradesMsg, UserEventsMsg]
+WsMsg = Union[AllMidsMsg, L2BookMsg, TradesMsg, UserEventsMsg, PongMsg]
+
+
+class Cloid:
+    def __init__(self, raw_cloid: str):
+        self._raw_cloid: str = raw_cloid
+        self._validate()
+
+    def _validate(self):
+        assert self._raw_cloid[:2] == "0x", "cloid is not a hex string"
+        assert len(self._raw_cloid[2:]) == 32, "cloid is not 16 bytes"
+
+    @staticmethod
+    def from_int(cloid: int):
+        return Cloid(f"{cloid:#034x}")
+
+    @staticmethod
+    def from_str(cloid: str):
+        return Cloid(cloid)
+
+    def to_raw(self):
+        return self._raw_cloid
